@@ -108,11 +108,20 @@ void PartMan::scan(DeviceItem *drvstart)
     const QJsonArray &jsonBD = jsonObjBD["blockdevices"].toArray();
 
     if (!drvstart) {
-        proc.exec("partprobe -s", true);
-        proc.exec("blkid -c /dev/null", true);
-        root.clear();
+        proc.exec("partprobe -s", true); // Inform the kernel of partition changes.
+        proc.exec("blkid -c /dev/null", true); // Refresh cached blkid information.
+    }
+    // Partitions listed in order of their physical locations.
+    QStringList order;
+    QString curdev;
+    for(const QString &line : proc.execOutLines("parted -lm", true)) {
+        const QString &sect = line.section(':', 0, 0);
+        const int part = sect.toInt();
+        if (part) order.append(DeviceItem::join(curdev, part));
+        else if (sect.startsWith("/dev/")) curdev = sect.mid(5);
     }
 
+    if (!drvstart) root.clear();
     for (const QJsonValue &jsonDrive : jsonBD) {
         const QString &jdName = jsonDrive["name"].toString();
         if (jsonDrive["type"] != "disk") continue;
@@ -125,7 +134,9 @@ void PartMan::scan(DeviceItem *drvstart)
         drvit->flags.curEmpty = true;
         drvit->flags.oldLayout = true;
         drvit->device = jdName;
-        drvit->flags.useGPT = (jsonDrive["pttype"]=="gpt");
+        const QString &ptType = jsonDrive["pttype"].toString();
+        drvit->flags.curGPT = (ptType=="gpt");
+        drvit->flags.curMBR = (ptType=="dos");
         drvit->size = jsonDrive["size"].toVariant().toLongLong();
         drvit->physec = jsonDrive["phy-sec"].toInt();
         drvit->curLabel = jsonDrive["label"].toString();
@@ -138,6 +149,7 @@ void PartMan::scan(DeviceItem *drvstart)
             drvit->flags.curEmpty = false;
             partit->flags.oldLayout = true;
             partit->device = jsonPart["name"].toString();
+            partit->order = order.indexOf(partit->device);
             partit->size = jsonPart["size"].toVariant().toLongLong();
             partit->physec = jsonPart["phy-sec"].toInt();
             partit->curLabel = jsonPart["label"].toString();
@@ -159,20 +171,27 @@ void PartMan::scan(DeviceItem *drvstart)
                 }
                 partit->flags.curESP = backup_list.contains(partit->device);
             }
-            if (partit->flags.curESP) partit->usefor = "ESP";
 
             partit->flags.bootRoot = (!bootUUID.isEmpty() && jsonPart["uuid"]==bootUUID);
             partit->curFormat = jsonPart["fstype"].toString();
             // Propagate the boot and nasty flags up to the drive.
             if (partit->flags.bootRoot) drvit->flags.bootRoot = true;
             if (partit->flags.nasty) drvit->flags.nasty = true;
-            notifyChange(partit);
+            else if (partit->flags.curESP && partit->allowedUsesFor().contains("ESP")) {
+                partit->usefor = "ESP";
+                partit->format = partit->allowedFormats().at(0);
+            }
         }
         for (int ixPart = drvit->childCount() - 1; ixPart >= 0; --ixPart) {
             // Propagate the boot flag across the entire drive.
             if (drvit->flags.bootRoot) drvit->child(ixPart)->flags.bootRoot = true;
         }
+        drvit->sortChildren();
         notifyChange(drvit);
+        // Hide the live boot media and its partitions by default.
+        if (!brave && drvit->flags.bootRoot) {
+            gui.treePartitions->setRowHidden(drvit->row(), QModelIndex(), true);
+        }
     }
 
     if (!drvstart) scanVirtualDevices(false);
@@ -223,7 +242,7 @@ void PartMan::scanVirtualDevices(bool rescan)
         DeviceItem *devit = listed.value(name);
         if (devit) {
             if (size != devit->size || physec != devit->physec || label != devit->curLabel
-                || crypto != devit->flags.cryptoV) {
+                || crypto != devit->flags.volCrypto) {
                 // List entry is different to the device, so refresh it.
                 delete devit;
                 devit = nullptr;
@@ -239,17 +258,29 @@ void PartMan::scanVirtualDevices(bool rescan)
             devit->flags.bootRoot = (!bootUUID.isEmpty() && jsonDev["uuid"]==bootUUID);
             devit->curLabel = label;
             devit->curFormat = jsonDev["fstype"].toString();
-            devit->flags.cryptoV = crypto;
-            notifyChange(devit);
-        }
-        DeviceItem *orit = findOrigin(name);
-        if (orit) {
-            orit->devMapper = name;
-            notifyChange(orit);
+            devit->flags.volCrypto = crypto;
+            devit->flags.oldLayout = true;
         }
     }
     for (const auto &it : listed.toStdMap()) delete it.second;
     vdlit->sortChildren();
+    for(int ixi = 0; ixi < vdlit->childCount(); ++ixi) {
+        const QString &name = vdlit->child(ixi)->device;
+        DeviceItem *vit = vdlit->child(ixi);
+        vit->origin = nullptr; // Clear stale origin pointer.
+        QStringList lines = proc.execOutLines("cryptsetup status " + vit->device, true);
+        // Find the associated partition and decrement its reference count if found.
+        for (const QString &line : lines) {
+            const QString &trline = line.trimmed();
+            if (!trline.startsWith("device:")) continue;
+            vit->origin = findDevice(trline.mid(trline.lastIndexOf('/') + 1));
+            if (vit->origin) {
+                vit->origin->devMapper = name;
+                notifyChange(vit->origin);
+                break;
+            }
+        }
+    }
     gui.treePartitions->expand(index(vdlit));
 }
 
@@ -260,21 +291,15 @@ bool PartMan::manageConfig(MSettings &config, bool save)
         DeviceItem *drvit = root.child(ixDrive);
         // Check if the drive is to be cleared and formatted.
         int partCount = drvit->childCount();
-        bool drvPreserve = !drvit->willInitPartTable();
+        bool drvPreserve = drvit->flags.oldLayout;
         const QString &configNewLayout = "Storage/NewLayout." + drvit->device;
-        QVariant bootdev;
         if (save) {
             if (!drvPreserve) config.setValue(configNewLayout, partCount);
-            if (drvit->active) {
-                bootdev = drvit->active->device;
-                config.setValue("Boot", bootdev);
-            }
         } else if (config.contains(configNewLayout)) {
             drvPreserve = false;
             drvit->clear();
             partCount = config.value(configNewLayout).toInt();
             if (partCount > PARTMAN_MAX_PARTS) return false;
-            bootdev = config.value("Boot");
         }
         // Partition configuration.
         const long long sizeMax = drvit->size - (PARTMAN_SAFETY_MB * 1048576);
@@ -283,7 +308,7 @@ bool PartMan::manageConfig(MSettings &config, bool save)
             DeviceItem *partit = nullptr;
             if (save || drvPreserve) {
                 partit = drvit->child(ixPart);
-                partit->flags.oldLayout = drvPreserve;
+                if (!save) partit->flags.oldLayout = true;
             } else {
                 partit = new DeviceItem(DeviceItem::Partition, drvit);
                 partit->device = DeviceItem::join(drvit->device, ixPart+1);
@@ -293,10 +318,11 @@ bool PartMan::manageConfig(MSettings &config, bool save)
             config.beginGroup(groupPart);
             if (save) {
                 config.setValue("Size", partit->size);
-                config.setValue("Encrypt", partit->encrypt);
-                if (partit->isActive()) config.setValue("Boot", true);
+                if (partit->isActive()) config.setValue("Active", true);
+                if (partit->addToCrypttab) config.setValue("AddToCrypttab", true);
                 if (!partit->usefor.isEmpty()) config.setValue("UseFor", partit->usefor);
                 if (!partit->format.isEmpty()) config.setValue("Format", partit->format);
+                config.setValue("Encrypt", partit->encrypt);
                 if (!partit->label.isEmpty()) config.setValue("Label", partit->label);
                 if (!partit->options.isEmpty()) config.setValue("Options", partit->options);
                 config.setValue("CheckBadBlocks", partit->chkbadblk);
@@ -307,18 +333,18 @@ bool PartMan::manageConfig(MSettings &config, bool save)
                     partit->size = config.value("Size").toLongLong();
                     sizeTotal += partit->size;
                     if (sizeTotal > sizeMax) return false;
-                    if (config.value("Boot").toBool()) partit->setActive(true);
+                    if (config.value("Active").toBool()) partit->setActive(true);
                 }
-                partit->usefor = config.value("UseFor").toString();
-                partit->format = config.value("Format").toString();
-                partit->chkbadblk = config.value("CheckBadBlocks").toBool();
-                partit->encrypt = config.value("Encrypt").toBool();
-                partit->label = config.value("Label").toString();
-                partit->options = config.value("Options").toString();
-                partit->dump = config.value("Dump").toBool();
-                partit->pass = config.value("Pass").toInt();
+                partit->addToCrypttab = config.value("AddToCrypttab").toBool();
+                partit->usefor = config.value("UseFor", partit->usefor).toString();
+                partit->format = config.value("Format", partit->format).toString();
+                partit->chkbadblk = config.value("CheckBadBlocks", partit->chkbadblk).toBool();
+                partit->encrypt = config.value("Encrypt", partit->encrypt).toBool();
+                partit->label = config.value("Label", partit->label).toString();
+                partit->options = config.value("Options", partit->options).toString();
+                partit->dump = config.value("Dump", partit->dump).toBool();
+                partit->pass = config.value("Pass", partit->pass).toInt();
             }
-            if (bootdev == partit->device) partit->setActive(true);
             int subvolCount = 0;
             if (partit->format == "btrfs") {
                 if (!save) subvolCount = config.value("Subvolumes").toInt();
@@ -426,12 +452,12 @@ void PartMan::treeMenu(const QPoint &)
         QAction *actUnlock = nullptr;
         QAction *actAddCrypttab = nullptr;
         QAction *actNewSubvolume = nullptr, *actScanSubvols = nullptr;
-        QAction *actSetBoot = nullptr;
+        QAction *actActive = nullptr;
         actRemove->setEnabled(gui.pushPartRemove->isEnabled());
         menu.addSeparator();
         if (twit->type == DeviceItem::VirtualBD) {
             actRemove->setEnabled(false);
-            if (twit->flags.cryptoV) actLock = menu.addAction(tr("&Lock"));
+            if (twit->flags.volCrypto) actLock = menu.addAction(tr("&Lock"));
         } else {
             bool allowCryptTab = false;
             if (twit->flags.oldLayout && twit->curFormat == "crypto_LUKS") {
@@ -442,13 +468,13 @@ void PartMan::treeMenu(const QPoint &)
             if (allowCryptTab) {
                 actAddCrypttab = menu.addAction(tr("Add to crypttab"));
                 actAddCrypttab->setCheckable(true);
-                actAddCrypttab->setChecked(twit->flags.autoCrypto);
+                actAddCrypttab->setChecked(twit->addToCrypttab);
                 actAddCrypttab->setEnabled(twit->willMap());
             }
             menu.addSeparator();
-            actSetBoot = menu.addAction(tr("Set boot flag"));
-            actSetBoot->setCheckable(true);
-            actSetBoot->setChecked(twit->isActive());
+            actActive = menu.addAction(tr("Active partition"));
+            actActive->setCheckable(true);
+            actActive->setChecked(twit->isActive());
         }
         if (twit->format == "btrfs") {
             menu.addSeparator();
@@ -463,10 +489,9 @@ void PartMan::treeMenu(const QPoint &)
         else if (action == actRemove) partRemoveClick(true);
         else if (action == actUnlock) partMenuUnlock(twit);
         else if (action == actLock) partMenuLock(twit);
-        else if (action == actSetBoot) twit->setActive(action->isChecked());
-        else if (action == actAddCrypttab) {
-            twit->flags.autoCrypto = action->isChecked();
-        } else if (action == actNewSubvolume) {
+        else if (action == actActive) twit->setActive(action->isChecked());
+        else if (action == actAddCrypttab) twit->addToCrypttab = action->isChecked();
+        else if (action == actNewSubvolume) {
             new DeviceItem(DeviceItem::Subvolume, twit);
             gui.treePartitions->expand(selIndex);
         } else if (action == actScanSubvols) {
@@ -495,8 +520,9 @@ void PartMan::treeMenu(const QPoint &)
         else if (action == actBasic) twit->layoutDefault(-1, false);
         else if (action == actCrypto) twit->layoutDefault(-1, true);
         else if (action == actReset) {
-            while (twit->childCount()) delete twit->child(0);
+            gui.boxMain->setEnabled(false);
             scan(twit);
+            gui.boxMain->setEnabled(true);
         }
     } else if (twit->type == DeviceItem::Subvolume) {
         QAction *actRemSubvolume = menu.addAction(tr("Remove subvolume"));
@@ -569,7 +595,7 @@ void PartMan::partMenuUnlock(DeviceItem *twit)
         const bool ok = luksOpen(twit->device, mapdev, editPass->text().toUtf8());
         if (ok) {
             twit->usefor.clear();
-            twit->flags.autoCrypto = checkCrypttab->isChecked();
+            twit->addToCrypttab = checkCrypttab->isChecked();
             twit->mapCount++;
             notifyChange(twit);
             scanVirtualDevices(true);
@@ -592,14 +618,15 @@ void PartMan::partMenuLock(DeviceItem *twit)
     const QString &dev = twit->mappedDevice();
     bool ok = false;
     // Find the associated partition and decrement its reference count if found.
-    DeviceItem *origin = findOrigin(dev);
+    DeviceItem *origin = twit->origin;
     if (origin) ok = proc.exec("cryptsetup close " + dev, true);
     if (ok) {
         if (origin->mapCount > 0) origin->mapCount--;
         origin->devMapper.clear();
-        origin->flags.autoCrypto = false;
+        origin->addToCrypttab = false;
         notifyChange(origin);
     }
+    twit->origin = nullptr;
     // Refresh virtual devices list.
     scanVirtualDevices(true);
     resizeColumnsToFit();
@@ -729,10 +756,8 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
         for (int ixdrv = 0; ixdrv < root.childCount(); ++ixdrv) {
             DeviceItem *drvit = root.child(ixdrv);
             const int partCount = drvit->childCount();
-            bool setupGPT = uefi || gptoverride || drvit->size >= 2199023255552 || partCount > 4;
-            if (!drvit->willInitPartTable()) {
-                setupGPT = drvit->flags.useGPT;
-            } else if (drvit->type != DeviceItem::VirtualDevices) {
+            bool setupGPT = drvit->willUseGPT();
+            if (!drvit->flags.oldLayout && drvit->type != DeviceItem::VirtualDevices) {
                 details += tr("Prepare %1 partition table on %2").arg(setupGPT?"GPT":"MBR", drvit->device) + '\n';
             }
             bool hasBiosGrub = false;
@@ -740,7 +765,7 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
                 DeviceItem *partit = drvit->child(ixdev);
                 const QString &use = partit->realUseFor();
                 QString actmsg;
-                if (!drvit->willInitPartTable()) {
+                if (drvit->flags.oldLayout) {
                     if (use.isEmpty()) continue;
                     else if (partit->willFormat()) actmsg = tr("Format %1 to use for %2");
                     else if (use != "/") actmsg = tr("Reuse (no reformat) %1 as %2");
@@ -776,30 +801,6 @@ bool PartMan::composeValidate(bool automatic, const QString &project)
         if (msgbox.exec() != QMessageBox::Yes) return false;
     }
 
-    calculatePartBD();
-    return true;
-}
-
-bool PartMan::calculatePartBD()
-{
-    listToUnmount.clear();
-    for (int ixDrive = 0; ixDrive < root.childCount(); ++ixDrive) {
-        DeviceItem *drvit = root.child(ixDrive);
-        if (drvit->type == DeviceItem::VirtualDevices) continue;
-        const bool useExist = !drvit->willInitPartTable();
-        const int partCount = drvit->childCount();
-        const long long driveSize = drvit->size / 1048576;
-        if (!useExist) {
-            // see if GPT needs to be used (either UEFI or >=2TB drive)
-            drvit->flags.useGPT = gptoverride || uefi || driveSize >= 2097152 || partCount > 4;
-        }
-        // Add (or mark) future partitions to the list.
-        for (int ixPart=0; ixPart < partCount; ++ixPart) {
-            DeviceItem *twit = drvit->child(ixPart);
-            if (twit->usefor.isEmpty()) continue;
-            if (useExist) listToUnmount << twit->device;
-        }
-    }
     return true;
 }
 
@@ -905,7 +906,7 @@ int PartMan::countPrepSteps()
     int nstep = 0;
     for (DeviceItemIterator it(*this); DeviceItem *item = *it; it.next()) {
         if (item->type == DeviceItem::Drive) {
-            if (item->willInitPartTable()) ++nstep; // New partition table
+            if (!item->flags.oldLayout) ++nstep; // New partition table
         } else if (item->isVolume()) {
             const QString &tuse = item->realUseFor();
             // Preparation
@@ -913,9 +914,11 @@ int PartMan::countPrepSteps()
             else if (!tuse.isEmpty()) ++nstep; // Existing partition
             // Formatting
             if (item->encrypt) nstep += 2; // LUKS Format
-            if (item->willFormat()) ++nstep; // New file system or subvolume
+            if (item->willFormat()) ++nstep; // New file system
             // Mounting
             if (tuse.startsWith('/')) ++nstep;
+        } else if (item->type == DeviceItem::Subvolume) {
+            ++nstep; // Create a new subvolume.
         }
     }
     return nstep;
@@ -925,7 +928,23 @@ bool PartMan::preparePartitions()
 {
     proc.log(__PRETTY_FUNCTION__);
 
-    // detach all existing partitions on the selected drive
+    // Detach all existing partitions.
+    QStringList listToUnmount;
+    for (int ixDrive = 0; ixDrive < root.childCount(); ++ixDrive) {
+        DeviceItem *drvit = root.child(ixDrive);
+        if (drvit->type == DeviceItem::VirtualDevices) continue;
+        const int partCount = drvit->childCount();
+        if (drvit->flags.oldLayout) {
+            // Using the existing layout, so only mark used partitions for unmounting.
+            for (int ixPart=0; ixPart < partCount; ++ixPart) {
+                DeviceItem *twit = drvit->child(ixPart);
+                if (!twit->usefor.isEmpty()) listToUnmount << twit->device;
+            }
+        } else {
+            // Clearing the drive, so mark all partitions on the drive for unmounting.
+            listToUnmount << proc.execOutLines("lsblk -nro name /dev/" + drvit->device, true);
+        }
+    }
     for (const QString &strdev : listToUnmount) {
         proc.exec("swapoff /dev/" + strdev, true);
         proc.exec("/bin/umount /dev/" + strdev, true);
@@ -934,7 +953,7 @@ bool PartMan::preparePartitions()
     // Prepare partition tables on devices which will have a new layout.
     for (int ixi = 0; ixi < root.childCount(); ++ixi) {
         DeviceItem *drvit = root.child(ixi);
-        if (!drvit->willInitPartTable()) continue;
+        if (drvit->flags.oldLayout || drvit->type == DeviceItem::VirtualDevices) continue;
         proc.status(tr("Preparing partition tables"));
 
         // Wipe the first and last 4MB to clear the partition tables, turbo-nuke style.
@@ -946,7 +965,7 @@ bool PartMan::preparePartitions()
         // Last 17KB = secondary partition table (for GPT disks).
         proc.exec(cmd + " seek=" + QString::number(offset));
         if (!proc.exec("parted -s /dev/" + drvit->device + " mklabel "
-            + (drvit->flags.useGPT ? "gpt" : "msdos"))) return false;
+            + (drvit->willUseGPT() ? "gpt" : "msdos"))) return false;
     }
     proc.sleep(1000);
 
@@ -956,8 +975,8 @@ bool PartMan::preparePartitions()
         DeviceItem *drvit = root.child(ixi);
         if (drvit->type == DeviceItem::VirtualDevices) continue;
         const int devCount = drvit->childCount();
-        const bool useGPT = drvit->flags.useGPT;
-        if (!drvit->willInitPartTable()) {
+        const bool useGPT = drvit->willUseGPT();
+        if (drvit->flags.oldLayout) {
             // Using existing partitions.
             QString cmd; // command to set the partition type
             if (useGPT) cmd = "/sbin/sgdisk /dev/%1 --typecode=%2:%3";
@@ -1031,7 +1050,10 @@ bool PartMan::formatPartitions()
         if (useFor == "FORMAT") proc.status(fmtstatus.arg(dev));
         else proc.status(fmtstatus.arg(twit->shownUseFor()));
         if (useFor == "ESP") {
-            if (!proc.exec("mkfs.msdos -F " + twit->format.mid(3)+' '+dev)) return false;
+            QString cmd("mkfs.msdos -F " + twit->format.mid(3));
+            if (twit->chkbadblk) cmd.append(" -c");
+            if (!twit->label.isEmpty()) cmd += " -n \"" + twit->label.trimmed().left(11) + '\"';
+            if (!proc.exec(cmd + ' ' + dev)) return false;
             // Sets boot flag and ESP flag.
             const QStringList &devsplit = DeviceItem::split(dev);
             if (!proc.exec("parted -s /dev/" + devsplit.at(0)
@@ -1150,7 +1172,7 @@ bool PartMan::fixCryptoSetup(const QString &keyfile, bool isNewKey)
     // Find extra devices
     QMap<QString, QString> extraAdd;
     for (DeviceItemIterator it(*this); DeviceItem *item = *it; it.next()) {
-        if (item->flags.autoCrypto) extraAdd.insert(item->device, item->mappedDevice());
+        if (item->addToCrypttab) extraAdd.insert(item->device, item->mappedDevice());
     }
     // File systems
     for (auto &it : mounts.toStdMap()) {
@@ -1331,23 +1353,6 @@ int PartMan::isEncrypt(const QString &point)
     return count;
 }
 
-DeviceItem *PartMan::findOrigin(const QString &vdev)
-{
-    QStringList lines = proc.execOutLines("cryptsetup status " + vdev, true);
-    // Find the associated partition and decrement its reference count if found.
-    for (const QString &line : lines) {
-        const QString &trline = line.trimmed();
-        if (trline.startsWith("device:")) {
-            const QString &trdev = trline.mid(trline.lastIndexOf('/') + 1);
-            for (DeviceItemIterator it(*this); *it; it.next()) {
-                if ((*it)->device == trdev) return *it;
-            }
-            return nullptr;
-        }
-    }
-    return nullptr;
-}
-
 DeviceItem *PartMan::findDevice(const QString &devname) const
 {
     for (DeviceItemIterator it(*this); *it; it.next()) {
@@ -1385,6 +1390,7 @@ QVariant PartMan::data(const QModelIndex &index, int role) const
         switch (index.column()) {
         case Device:
             if (item->type == DeviceItem::Subvolume) return "----";
+            else if (item->isActive()) return item->device + '*';
             else return item->device;
             break;
         case Size:
@@ -1400,8 +1406,13 @@ QVariant PartMan::data(const QModelIndex &index, int role) const
             else return item->curLabel;
             break;
         case Format:
-            if (item->usefor.isEmpty()) return item->curFormat;
-            else return item->shownFormat(item->format);
+            if (item->type == DeviceItem::Drive) {
+                if (item->willUseGPT()) return "GPT";
+                else if (item->flags.curMBR || !item->flags.oldLayout) return "MBR";
+            } else {
+                if (item->usefor.isEmpty()) return item->curFormat;
+                else return item->shownFormat(item->format);
+            }
             break;
         case Options:
             if (item->canMount() || item->realUseFor() == "SWAP") return item->options;
@@ -1429,15 +1440,11 @@ QVariant PartMan::data(const QModelIndex &index, int role) const
     } else if (role == Qt::DecorationRole && index.column() == Device) {
         if (item->type == DeviceItem::Drive && !item->flags.oldLayout) {
             return QIcon(":/appointment-soon");
-        } else if (item->type == DeviceItem::VirtualBD && item->flags.cryptoV) {
+        } else if (item->type == DeviceItem::VirtualBD && item->flags.volCrypto) {
             return QIcon::fromTheme("unlock");
         } else if (item->mapCount) {
             return QIcon::fromTheme("lock");
         }
-    } else if (role == Qt::FontRole && index.column() == Device) {
-        QFont font;
-        font.setItalic(item->isActive());
-        return font;
     }
     return QVariant();
 }
@@ -1471,7 +1478,7 @@ Qt::ItemFlags PartMan::flags(const QModelIndex &index) const
         }
         break;
     case UseFor:
-        if (item->allowedUsesFor().count() > 1) flagsOut |= Qt::ItemIsEditable;
+        if (item->allowedUsesFor().count() >= 1) flagsOut |= Qt::ItemIsEditable;
         break;
     case Label:
         if (item->type == DeviceItem::Subvolume) {
@@ -1487,7 +1494,8 @@ Qt::ItemFlags PartMan::flags(const QModelIndex &index) const
         if (item->allowedFormats().count() > 1) flagsOut |= Qt::ItemIsEditable;
         break;
     case Check:
-        if (item->format.startsWith("ext") || item->format == "jfs") {
+        if (item->format.startsWith("ext") || item->format == "jfs"
+            || item->format.startsWith("FAT", Qt::CaseInsensitive)) {
             flagsOut |= Qt::ItemIsUserCheckable;
         }
         break;
@@ -1620,10 +1628,9 @@ int PartMan::changeEnd(bool notify)
 }
 void PartMan::notifyChange(class DeviceItem *item, int first, int last)
 {
-    DeviceItem *const p = item->parentItem;
     if (first < 0) first = 0;
-    if (last < 0) last = p ? p->childCount() - 1 : 0;
-    const int row = p ? p->indexOfChild(item) : 0;
+    if (last < 0) last = _TreeColumns_ - 1;
+    const int row = item->row();
     emit dataChanged(createIndex(row, first, item), createIndex(row, last, item));
 }
 
@@ -1634,6 +1641,8 @@ DeviceItem::DeviceItem(enum DeviceType type, DeviceItem *parent, DeviceItem *pre
 {
     if (type == Partition) size = 1048576;
     if (parent) {
+        if (type == Subvolume) size = parent->size;
+        physec = parent->physec;
         partman = parent->partman;
         const int i = preceding ? (parentItem->children.indexOf(preceding) + 1) : parentItem->childCount();
         if (partman) partman->beginInsertRows(partman->index(parent), i, i);
@@ -1680,7 +1689,7 @@ inline int DeviceItem::row() const
 {
     return parentItem ? parentItem->children.indexOf(const_cast<DeviceItem *>(this)) : 0;
 }
-inline DeviceItem *DeviceItem::parent() const
+DeviceItem *DeviceItem::parent() const
 {
     if (parentItem && !parentItem->parentItem) return nullptr; // Invisible root
     return parentItem;
@@ -1701,6 +1710,7 @@ inline int DeviceItem::childCount() const
 void DeviceItem::sortChildren()
 {
     auto cmp = [](DeviceItem *l, DeviceItem *r) {
+        if (l->order != r->order) return l->order < r->order;
         return l->device < r->device;
     };
     std::sort(children.begin(), children.end(), cmp);
@@ -1748,6 +1758,14 @@ bool DeviceItem::isLocked() const
     }
     return (mapCount != 0);
 }
+inline bool DeviceItem::willUseGPT() const
+{
+    if (type != Drive) return false;
+    if (flags.oldLayout) return flags.curGPT;
+    else if (size >= 2199023255552 || children.count() > 4) return true;
+    else if (partman) return (partman->gptoverride || partman->uefi);
+    return false;
+}
 inline bool DeviceItem::willFormat() const
 {
     return format != "PRESERVE" && !usefor.isEmpty();
@@ -1784,7 +1802,7 @@ QString DeviceItem::shownDevice() const
 }
 QStringList DeviceItem::allowedUsesFor(bool real) const
 {
-    if (!isVolume()) return QStringList();
+    if (!isVolume() && type != Subvolume) return QStringList();
     QStringList list;
     if (!partman || size >= partman->rootSpaceNeeded) list << "root";
     if (type == Subvolume) list << "home"; // swap requires Linux 5.0 or later
@@ -1792,7 +1810,7 @@ QStringList DeviceItem::allowedUsesFor(bool real) const
         list.prepend("Format");
         if (type != VirtualBD) {
             if (size <= 16777216) list << "BIOS-GRUB";
-            if (size <= 4294967296) list << "ESP" << "boot";
+            if (size <= 8589934592) list << "ESP" << "boot";
         }
         list << "swap" << "home";
     }
@@ -1803,21 +1821,22 @@ QStringList DeviceItem::allowedUsesFor(bool real) const
 }
 QStringList DeviceItem::allowedFormats() const
 {
-    if (!isVolume()) return QStringList();
     QStringList list;
     bool allowPreserve = false, selPreserve = false;
-    if (type == Subvolume) list << "CREATE";
-    else {
+    if (type == Subvolume) list.append("CREATE");
+    else if (isVolume()) {
         const QString &use = realUseFor();
         if (use.isEmpty()) return QStringList();
-        else if (use == "/boot") list << "ext4";
-        else if (use == "BIOS-GRUB") list << "GRUB";
+        else if (use == "/boot") list.append("ext4");
+        else if (use == "BIOS-GRUB") list.append("GRUB");
         else if (use == "ESP") {
-            list << "FAT32" << "FAT16" << "FAT12";
+            list.append("FAT32");
+            if (size <= 4294901760) list.append("FAT16");
+            if (size <= 33553920) list.append("FAT12");
             selPreserve = allowPreserve = (list.contains(curFormat, Qt::CaseInsensitive)
                 || !curFormat.compare("VFAT", Qt::CaseInsensitive));
         } else if (use == "SWAP") {
-            list << "SWAP";
+            list.append("SWAP");
             selPreserve = allowPreserve = list.contains(curFormat, Qt::CaseInsensitive);
         } else {
             list << "ext4" << "ext3" << "ext2";
@@ -1874,21 +1893,14 @@ DeviceItem *DeviceItem::addPart(int defaultMB, const QString &defaultUse, bool c
 void DeviceItem::driveAutoSetActive()
 {
     if (active) return;
-    const int partcount = children.count();
+    if (partman && partman->uefi && willUseGPT()) return;
+    // Cannot use partman->mounts map here as it may not be populated.
     for (const QString &pref : QStringList({"/boot", "/"})) {
-        for (int ixPart = 0; ixPart < partcount; ++ixPart) {
-            DeviceItem *partit = child(ixPart);
-            if (partit->realUseFor() == pref) {
-                partit->setActive(true);
+        for (DeviceItemIterator it(this); DeviceItem *item = *it; it.next()) {
+            if (item->realUseFor() == pref) {
+                while (item && item->type != Partition) item = item->parentItem;
+                if (item) item->setActive(true);
                 return;
-            } else {
-                const int svcount = partit->childCount();
-                for (int ixSV = 0; ixSV < svcount; ++ixSV) {
-                    if (partit->child(ixSV)->realUseFor() == pref) {
-                        partit->setActive(true);
-                        return;
-                    }
-                }
             }
         }
     }
@@ -1925,7 +1937,9 @@ void DeviceItem::autoFill(unsigned int changed)
         }
         // Automatic default boot device selection
         if ((type != VirtualBD) && (use == "/boot" || use == "/")) {
-            if (parentItem) parentItem->driveAutoSetActive();
+            DeviceItem *drvit = this;
+            while (drvit && drvit->type != Drive) drvit = drvit->parentItem;
+            if (drvit) drvit->driveAutoSetActive();
         }
     }
     if (format.isEmpty()) {
