@@ -43,12 +43,14 @@ class DeviceItem
     QVector<DeviceItem *> children;
     DeviceItem *parentItem = nullptr;
     class PartMan *partman = nullptr;
+    int order = -1;
     DeviceItem *addPart(int defaultMB, const QString &defaultUse, bool crypto);
     void driveAutoSetActive();
     void autoFill(unsigned int changed = 0xFFFF);
     void labelParts();
 public:
     DeviceItem *active = nullptr;
+    DeviceItem *origin = nullptr;
     enum DeviceType {
         Unknown,
         Drive,
@@ -62,23 +64,25 @@ public:
         bool curEmpty : 1;
         bool oldLayout : 1;
         bool bootRoot : 1;
-        bool useGPT : 1;
+        bool curMBR : 1;
+        bool curGPT : 1;
         bool curESP : 1;
-        bool cryptoV : 1;
-        bool autoCrypto : 1;
+        bool volCrypto : 1;
     } flags = {};
-    int mapCount = 0;
-    int physec;
     QString model, device, devMapper;
-    long long size = 0;
     QString label, curLabel;
     QString usefor;
-    bool encrypt = false;
     QString format, curFormat;
-    bool chkbadblk = false;
     QString options;
-    bool dump = false;
+    long long size = 0;
+    int mapCount = 0;
+    int physec;
     int pass = 0;
+    bool encrypt = false;
+    bool chkbadblk = false;
+    bool dump = false;
+    bool addToCrypttab = false;
+
     DeviceItem(enum DeviceType type, DeviceItem *parent = nullptr, DeviceItem *preceding = nullptr);
     ~DeviceItem();
     void clear();
@@ -95,7 +99,7 @@ public:
     void setActive(bool boot);
     bool isActive() const;
     bool isLocked() const;
-    inline bool willInitPartTable() const { return (type == Drive && (!flags.oldLayout || flags.curEmpty)); }
+    bool willUseGPT() const;
     bool willFormat() const;
     bool canEncrypt() const;
     QString mappedDevice(const bool full = false) const;
@@ -150,15 +154,11 @@ class PartMan : public QAbstractItemModel
     DeviceItem *changing = nullptr;
     Ui::MeInstall &gui;
     QWidget *master;
-    QMap<QString, DeviceItem *> mounts;
-    QStringList listToUnmount;
     void setup();
     void scanVirtualDevices(bool rescan);
     void resizeColumnsToFit();
     bool formatLinuxPartition(const QString &devpath, const QString &format, bool chkBadBlocks, const QString &label);
-    bool calculatePartBD();
     bool prepareSubvolumes(DeviceItem *partit);
-    DeviceItem *findOrigin(const QString &vdev);
     void treeItemChange();
     void treeSelChange();
     void treeMenu(const QPoint &);
@@ -190,6 +190,7 @@ public:
     bool gptoverride=false, uefi=false, brave=false;
     long long rootSpaceNeeded = 0;
     long long bootSpaceNeeded = 0;
+    QMap<QString, DeviceItem *> mounts;
     QMap<QString, QString> defaultLabels;
     PartMan(MProcess &mproc, Ui::MeInstall &ui, QWidget *parent);
     void scan(DeviceItem *drvstart = nullptr);
